use Test::More tests => 37;

use strict;
use warnings;

use Text::CSV_XS;
use Encode qw( encode_utf8 );
use DE_EPAGES::Core::API::CSV qw( CombineCSV ParseCSV CombineCSVArray ParseCSVArray );
use DE_EPAGES::Core::API::CSV::BaseExportDriver;
use DE_EPAGES::Core::API::File qw ( DeleteFile );
use DE_EPAGES::Core::API::Script qw( RunScript );

##no critic qw(ProhibitInterpolationOfLiterals)
RunScript( Message => 'long', Sub => sub {
    my $csv = Text::CSV_XS->new({ 'eol'=>"\n", 'binary'=>1 });
    my @SampleFields = ('You said, "Hello, Regine!"',
                 5.67,
                 "Surely \n Iam here",
                 '',
                 '3.14159',);

    my $String = "\"You said, \"\"Hello, Regine!\"\"\",5.67,\"Surely \n Iam here\",,3.14159\n";
    if ($csv->combine(@SampleFields)) {
        ok ($String eq $csv->string, 'string correct');
    } else {
        ok (0, $csv->error_input);
    }

    if ($csv->parse($String)) {
        my @Fields = $csv->fields;
        my $AllOK = 1;
        for (my $i=0;$i<scalar @SampleFields;$i++) {
            $AllOK = 0 unless $SampleFields[$i] eq $Fields[$i];
        }
        ok ($AllOK, 'all fields equal');
    } else {
        ok (0, $csv->error_input);
    }

    #
    # DE_EPAGES::Core::API::CSV::BaseExportDriver
    # DE_EPAGES::Core::API::CSV::BaseImportHandler
    #

    my $Exporter = DE_EPAGES::Core::API::CSV::BaseExportDriver->new;
    $Exporter->exportCSV('test.csv', undef, [ qw (ID Name Relative)], [
        ['1234','Thomas','Ich'],
        ['5678','Regine','Du'],
        ['9012','epages','Wir'],
    ]);

    my $Importer = MyImportHandler->importCSV('test.csv');
    is( scalar @{$Importer->{'Items'}}, 3, 'importCSV: number of lines' );
    is( $Importer->header->[2], 'Relative', 'importCSV: header' );
    is( $Importer->{'Items'}->[1]->{'Name'}, 'Regine', 'importCSV: data' );

    $Exporter->exportCSV('test.csv', { 'open_mode' => '>>' }, [ qw (ID Name Relative)], [
        ['3854','Christian','Er'],
        ['3423','Christiane','Sie'],
    ]);

    $Importer = MyImportHandler->importCSV('test.csv');
    is( scalar @{$Importer->{'Items'}}, 5, 'exportCSV: appended 2 lines without new header' );

    DeleteFile('test.csv');

    #
    # DE_EPAGES::Core::API::CSV
    #

    ok( ',,' eq CombineCSV( [ '', '', '' ] ), 'combine empty strings' );
    ok( ',,' eq CombineCSV( [ undef, '', undef ] ), 'combine undefined strings' );
    ok( 3 == scalar @{ ParseCSV( ',,' ) }, 'parse empty strings' );
    ok( defined ParseCSV( ',,' )->[2], 'parsed empty value is defined' );
    ok( 1 == scalar @{ ParseCSV( '' ) }, 'parse empty string' );
    ok( 1 == scalar @{ ParseCSV( undef ) }, 'parsed undefined' );
    ok( 1 == scalar @{ ParseCSV( ' ' ) }, 'parse space' );

    $String = encode_utf8( '////' );
    my $aFields = ParseCSV( $String, { sep_char => '/' } );
    ok( 5 == scalar @$aFields, 'parse umlauts' );
    ok( $aFields->[2] eq encode_utf8(''), 'parse umlauts 2' );
    ok( encode_utf8( '/""/""/""/' ) eq CombineCSV( $aFields, { sep_char => '/' } ), 'combine umlauts' );

    $csv = DE_EPAGES::Core::API::CSV->new( { sep_char => ';' } );
    ok( $csv->converter->isa( 'Text::CSV_XS' ), 'converter' );
    ok( 'a;" b b ";c' eq $csv->combine([ 'a', ' b b ', 'c' ]), 'combine' );
    ok( ' b b ' eq $csv->parse( 'a;" b b ";c' )->[1], 'parse' );
    ok( " \r\n...\n " eq $csv->parse( "a;\" \r\n...\n \";c" )->[1], 'parse newline' );
    ok( ';' eq $csv->parse( "a;\";\";c" )->[1], 'parse separator' );
    ok( '";"' eq $csv->combine([ ';' ]), 'combine separator' );
    ok( "\"\n\"" eq $csv->combine([ "\n" ]), 'combine newline' );
    ok( '""""' eq $csv->combine([ '"' ]), 'combine quote' );

    # multiline
    $csv = DE_EPAGES::Core::API::CSV->new( { sep_char => ';', eol => "\n" } );
    ok( "a;b\n" eq $csv->combine([ 'a', 'b' ]), 'multiline combine' );
    ok( "\"\n\"\n" eq $csv->combine([ "\n" ]), 'multiline combine newline' );

    # always_quote
    $csv = DE_EPAGES::Core::API::CSV->new( { sep_char => ';', always_quote => '"' } );
    is( $csv->combine([ 'a', 'b;c', 'say "cheese"' ]), '"a";"b;c";"say ""cheese"""', 'always_quote' );

    # CombineCSVArray, ParseCSVArray
    my $Content = "1,\"\"\"2\",3\n4,\"5\n5\",6\n";
    my $Combined = CombineCSVArray( [ [ 1,'"2',3 ], [4,"5\n5",6] ] );
    # print "'$Combined'\n";
    # print "'$Content'\n";
    ok( $Content eq $Combined, 'CombineCSVArray with defaults' );

    my $aLines = ParseCSVArray( $Content );
    ok( 2 == @$aLines, 'ParseCSVArray - number of lines' );
    ok( '"2' eq $aLines->[0]->[1], 'ParseCSVArray - [0][1]' );
    ok( "5\n5" eq $aLines->[1]->[1], 'ParseCSVArray - [1][1]' );
    ok( '6' eq $aLines->[1]->[2], 'ParseCSVArray - [1][2]' );

    my %Options = ( sep_char => ';', quote_char => "'", escape_char => "'" );
    $Content = "1;'''2';3\n4;'5\n5';6\n";
    $Combined = CombineCSVArray( [ [ 1,"'2",3 ], [4,"5\n5",6] ], \%Options );
    ok( $Content eq $Combined, 'CombineCSVArray with special delimiters' );

    $aLines = ParseCSVArray( $Content, \%Options );
    ok( 2 == @$aLines, 'ParseCSVArray with special delimiters - number of lines' );
    ok( "'2" eq $aLines->[0]->[1], 'ParseCSVArray - [0][1]' );
    ok( "5\n5" eq $aLines->[1]->[1], 'ParseCSVArray - [1][1]' );
    ok( '6' eq $aLines->[1]->[2], 'ParseCSVArray - [1][2]' );
});

# helper package for
# DE_EPAGES::Core::API::CSV::BaseImportHandler

package MyImportHandler;
use base qw( DE_EPAGES::Core::API::CSV::BaseImportHandler );
use strict;

sub new {
    my $class = shift;
    my $self = $class->SUPER::new(@_);

    $self->{'Items'} = [];
    return $self;
}

sub importLine {
    my $self = shift;

    my $aHeader = $self->header;
    my %Columns;
    @Columns{@$aHeader} = @{$self->topID('Item')};
    push @{$self->{'Items'}}, \%Columns;
    return;
}
