/*
 * @Copyright   Copyright 2006-2013, epages GmbH, All Rights Reserved.
 *
 * @module      ckeditor/plugins/epgadget/plugin
 *
 * @revision    $$
 */
/*global define, window, de_epages*/
define('ckeditor/plugins/epgadget/plugin', [
    'jquery',
    'ep',
    'ckeditor/ckeditor',
    'util/json',
    'de_epages/externalcontent/gadgetdialog/constants',
    '$dict!ep/dict'
], function ($, ep, CKEDITOR, JSON, CONSTANTS, epDict) {
    'use strict';

    var selectedNode,
        tGadget = epDict.translate('Gadget'),
        topRequire = window.require,
        gadgetClass = CONSTANTS.gClassname,
        pluginName = 'epgadget',
        // get the correct image source for each gadget
        createImgSrc = function (data) {
            var srcValue = ep.config.iconsRoot + '/gadget',
                opt;

            if (data) {
                srcValue += '_' + data.alias;
                opt = data.options;
                if (opt && data.alias === 'maps') {
                    if (opt.layout) {
                        srcValue += '_' + opt.layout;
                    }
                    if (opt.type) {
                        srcValue += '_' + opt.type;
                    }
                }
            }

            srcValue += '.png';
            return srcValue;
        },
        // remove paragraphs around gadget elements if the are inserted (htmlFilter)
        removeParagraphsAroundGadgetElement = {
            elements : {
                p: function (element) {
                    var child = element.children[0],
                        pattern = /\bep-gadget\b/;      // regular expression to indicate gadget DIVs by class name

                    // is p tag empty?
                    if (child && !child.children && (!child.value || CKEDITOR.tools.trim(child.value).match(/^(?:&nbsp;|\xa0|<br \/>)$/))) {
                        // is previous or next element a gadget div -> remove empty p tag
                        if ((element.previous && element.previous.attributes && element.next && pattern.test(element.previous.attributes['class'])) || (element.next && element.next.attributes && pattern.test(element.next.attributes['class']))) {
                            return false;
                        }
                    }

                    return element;
                }
            }
        };

    // Work-around for opening dialog outside of iframe,
    // when CKEditor is instanciated on the SF inside an iframe in MBO.
    if (window.top !== window.self) {
        try {
            topRequire = window.top.require;
        } catch (ignore) {}
    }


    function insertGadget(editor, element) {
        editor.focus();

        var selection = editor.getSelection(),
            ranges = selection.getRanges(),
            range = ranges[0],
            position;

        if ((range.checkStartOfBlock() && range.checkEndOfBlock()) || range.checkStartOfBlock()) {
            position = CKEDITOR.POSITION_BEFORE_START;
        } else if (range.checkEndOfBlock()) {
            position = CKEDITOR.POSITION_AFTER_END;
        }

        editor.insertElement(element);

        if (position) {
            window.setTimeout(function() {
                selection = editor.getSelection();
                range = selection.getRanges()[0];

                var pCon = range.startContainer.getAscendant({p: 1}, true);

                if (pCon !== null) {
                    element[position === CKEDITOR.POSITION_BEFORE_START ? 'insertBefore' : 'insertAfter'](pCon);
                }
            }, 1);
        }
    }


    CKEDITOR.plugins.add(pluginName, {

        init: function (editor) {
			// Handling for special cases:
			// Insert empty p-tag if only one gadget is inserted.
            editor.on( 'toDataFormat', function( evt ) {
                var gadgetfragment = $('<div/>').html(evt.data.dataValue);
                if (gadgetfragment.children('div.ep-gadget') && gadgetfragment.children().length === 1) {
                    evt.data.dataValue += '<p>&nbsp;</p>';
                }
            }, null, null, 1 );

            editor.addCommand(pluginName, {
                exec: function (editor, alias) {
                    var selectedElement, dataAttribute, gadgetOptions = {
                        callback: function (data) {
                            var imgSrc = createImgSrc(data),
                                element = new CKEDITOR.dom.element('div'),
                                dataString = JSON.stringify(data),
                                fakeElement;

                            element
                                .setAttributes({
                                    'class': gadgetClass,
                                    'data-ep-gadget': dataString
                                })
                                .setText(' ');

                            // Create *fakeElement* which is used as placeholder.
                            fakeElement = editor.createFakeElement(element, gadgetClass, 'div');
                            fakeElement
                                .setAttributes({
                                    'alt': tGadget,
                                    'title': tGadget,
                                    'data-ep-gadget': dataString,
                                    'contenteditable': false,
                                    'src': imgSrc
                                })
                                .setStyles({
                                    'max-width': '100%',
                                    'display': 'block',
                                    'margin-bottom': '10px'
                                });

                            // Remove selected element if gadget was selected before.
                            if (selectedNode) {
                                selectedElement = selectedNode.getSelectedElement();
                                if (selectedElement) {
                                    selectedElement.remove();
                                }
                            }

                            insertGadget(editor, fakeElement);
                        }
                    };
                    // Check if gadget is selected and get gadget data from *fakeElement*.
                    selectedNode = editor.getSelection();
                    if (selectedNode) {
                        selectedElement = selectedNode.getSelectedElement();
                        if (selectedElement) {
                            dataAttribute = selectedElement.getAttribute('data-ep-gadget');
                            if (dataAttribute) {
                                gadgetOptions.gadget = JSON.parse(dataAttribute);
                            }
                        }
                    }

                    if (alias && alias === 'rss' && !gadgetOptions.gadget) {
                        gadgetOptions.gadget = {
                            alias: 'rss'
                        };
                    }

                    topRequire(['de_epages/externalcontent/gadgetdialog'], function (gadgetsdialog) {
                        // with EPG-28229 we removed the types "profile" and "search"
                        // but it is possible that merchants still have these types in their shop
                        // so we will send them to the standard twitter gadget, when they edit the old profile/search-twitter gadget.
                        if (gadgetOptions.gadget && gadgetOptions.gadget.alias === 'twitter' && (gadgetOptions.gadget.options.type === 'profile' || gadgetOptions.gadget.options.type === 'search')) {
                            gadgetOptions.gadget.options = {};
                        }
                        gadgetsdialog.open(gadgetOptions);
                    });
                }
            });

            editor.ui.addButton(pluginName, {
                label: tGadget,
                command: pluginName,
                icon: this.path + 'images/gadget.png',
                onRender: function () {
                    // Highlight gadget button on gadget selection.
                    editor.on('selectionChange', function (ev) {
                        this.setState(CKEDITOR.TRISTATE_OFF);

                        if (ev.data && ev.data.selection && ev.data.selection.getSelectedElement() !== null && ev.data.selection.getSelectedElement().hasClass(gadgetClass)) {
                            this.setState(CKEDITOR.TRISTATE_ON);
                        }
                    }, this);
                }
            });

            editor.on('doubleclick', function (ev) {
                if (ev.data && ev.data.element && ev.data.element.hasClass(gadgetClass)) {
                    editor.execCommand(pluginName);
                }
            });

        },

        afterInit: function (editor) {
            var dataProcessor = editor.dataProcessor,
                dataFilter = dataProcessor && dataProcessor.dataFilter,
                htmlFilter = dataProcessor && dataProcessor.htmlFilter;

            if (dataFilter) {
                dataFilter.addRules({
                    elements: {
                        div: function (element) {
                            // Select gadget and replace it with *fakeElement*.
                            if (element.attributes['class'] === gadgetClass) {
                                var fakeElement = editor.createFakeParserElement(element, gadgetClass, 'div', false),
                                    dataString = $('<div/>').html(element.attributes['data-ep-gadget']).text(),
                                    data = JSON.parse(dataString),
                                    imgSrc = createImgSrc(data),
                                    styles = [
                                        "max-width: 100%",
                                        "display: block",
                                        "margin-bottom: 10px"
                                    ];

                                // We have to use fakeElement.attributes here because there is no setAttributes method for a FakeParserElement - very strange.
                                fakeElement.attributes.title = tGadget;
                                fakeElement.attributes.alt = tGadget;
                                fakeElement.attributes.style = styles.join(';');
                                fakeElement.attributes.src = imgSrc;
                                fakeElement.attributes.contenteditable = false;
                                fakeElement.attributes['data-ep-gadget'] = dataString;

                                return fakeElement;
                            }
                        },
                        p: function (element) {
                            var child = element.children[0],
                                pattern = /\bep-gadget\b/;      // regular expression to indicate gadget DIVs by class name

                            // is p tag empty?
                            if (child && !child.children && (!child.value || CKEDITOR.tools.trim(child.value).match(/^(?:&nbsp;|\xa0|<br \/>)$/))) {
                                // is previous or next element a gadget div -> remove empty p tag
                                if ((element.previous && element.previous.attributes && element.next && pattern.test(element.previous.attributes['class'])) || (element.next && element.next.attributes && pattern.test(element.next.attributes['class']))) {
                                    return false;
                                }
                            }

                            return element;
                        }
                    }
                }, 100);
            }


            if (htmlFilter) {
                // add rule to get rid of p-tags around gadget elements
                htmlFilter.addRules(removeParagraphsAroundGadgetElement, 100);
            }
        }
    });

    return CKEDITOR;
});