﻿use strict;
use Test::More tests => 24;

## no critic qw( ProhibitLongLines )

use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Object::API::Factory qw( ExistsObjectByPath LoadObjectByPath LoadRootObject LoadClassByAlias ExistsClassByAlias );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite GetCurrentDBHandle );

my $ATTRIBUTE_PACKAGE = 'DE_EPAGES::Object::API::Attributes::DefaultAttribute';
my $ROOTPATH = '/Test';

sub Main {
    ConnectTestSite();

    my $DriverName = GetCurrentDBHandle->driverName;

    # create test classes
    ok( Init(), 'initialization' );

    my $TestObject = LoadObjectByPath( $ROOTPATH );
    my $Object = LoadClassByAlias('C1')->insertObject({
        Parent => $TestObject,
        Alias => 'C1O1'
    } );
    my $ObjectID = $Object->id;
    ok( $ObjectID, 'InsertObject' );


    my $Description = 'a asdas weq wrwe rewr werwe rwer werwerw er werw erweeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr tttttttttttttttttttttttttttttttttttttt rerrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww ende';
    $Object->set({'FirstName'=>'Wolfgang'});
    $Object->set({
        LastName => 'Loch',
        Description => $Description
    });

    # GetAttributes
    ok( $Object->get('FirstName') eq 'Wolfgang', 'short string value 1' );
    ok( $Object->get('LastName') eq 'Loch', 'short string value 2' );
    ok( $Object->get('Description') eq $Description, 'long string value' );

    # empty string
    ok( (not defined $Object->get('A1String')), 'empty string is not defined' );

    $Object->set({'A1String'=>'Test'});
    ok( $Object->get('A1String' ) eq 'Test', 'update empty string' );

    $Object->set({'A1String'=>''});
    ok( (not defined $Object->get('A1String')), 'make non-empty string empty' );

    $Object->set({'A1String'=>'Test'});
    $Object->set({'A1String'=>undef});
    ok( (not defined $Object->get('A1String')), 'set string to undef' );

    # spaces
    $Object->set({'A1String'=>' '});
    is( $Object->get('A1String' ), undef, 'single space' );

    $Object->set({'A1String'=>' -'});
    is( $Object->get('A1String' ), ' -', 'leading space' );

    $Object->set({'A1String'=>'- '});
    if( $DriverName eq 'Sybase' ) {
        is( $Object->get('A1String' ), '-', 'trailing space (string attribute)' );
    }
    else {
        is( $Object->get('A1String' ), '- ', 'trailing space (string attribute)' );
    }

    my $sentence = 'This is a somewhat long sentence with some nonsense but a trailing whitespace at the end which should be trimmed by buggy sybase databases erweeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr tttttttttttttttttttttttttttttttttttttt rerrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww';
    $Object->set({'A1String'=> $sentence . ' '});
    ok( $Object->get('A1String' ) eq "$sentence ", 'trailing space (text attribute)  -> EPG 13044' );

    $Object->set({'A1String'=>'  '});
    is( $Object->get('A1String'), undef, 'multiple spaces (string attribute)' );

    $sentence = 'This is a somewhat long sentence with some nonsense but a trailing whitespace at the end which should be trimmed by buggy sybase databases erweeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr tttttttttttttttttttttttttttttttttttttt rerrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww';
    $Object->set({'A1String'=> $sentence . '    '});
    ok( $Object->get('A1String' ) eq "$sentence    ", 'multiple spaces (text attribute) -> EPG 13044' );

    $sentence = 'X' x 65600; ## no critic qw(ProhibitMagicNumbers)
    $Object->set({'A1String'=> $sentence });
    ok( $Object->get('A1String' ) eq "$sentence", 'long string (EPG-14094)' );

    # UTF-8
    use utf8;
    my $ShortString = 'Радуга позволяет создавать';
    my $LongString = 'Радуга позволяет создавать плейлисты простым перемещением файлов из окна Проводника в главное окно Радуги. Вы можете изменять порядок проигрывания треков, перетаскивая их мышкой. Как только плейлист составлен, можно выходить в эфир. Просто нажмите кнопку \'Воспроизведение\' - и Вы в эфире!';
    no utf8;

    $Object->set({'A1String'=>$ShortString});
    ok( $Object->get('A1String' ) eq $ShortString, 'short UTF-8 string' );

    $Object->set({'A1String'=>$LongString});
    ok( $Object->get('A1String' ) eq $LongString, 'long UTF-8 string' );

    # ExternalPassword
    my $Class = LoadClassByAlias('C1');
    my $Attribute = $Class->insertAttribute({ Alias => 'ExtPwd', Type => 'ExternalPassword', Package => $ATTRIBUTE_PACKAGE });
    is( $Attribute->get('Type'), 'ExternalPassword', 'ExternalPassword: data type' );

    $Object->set({'ExtPwd' => 'geheim'});
    is( $Object->get('ExtPwd'), 'geheim', 'ExternalPassword: set/get' );

    $Class->deleteAttribute('ExtPwd');
    ok( !$Class->existsAttribute('ExtPwd'), 'attribute is deleted' );
    eval { $Object->get('ExtPwd') };
    ok( ExistsError(), 'get non-existing attribute' );
    is( GetError->code, 'NoResults_Attribute', 'can\'t get non-existing attribute' );

    $Object->delete;

    ok( Cleanup(), 'cleanup' );

    DisconnectSite();
}

sub Cleanup {
    if( ExistsClassByAlias( 'C1' ) ) {
        LoadClassByAlias( 'C1' )->delete;
    }
    if( ExistsObjectByPath( $ROOTPATH ) ) {
        LoadObjectByPath( $ROOTPATH )->delete;
    }
    return 1;
}

sub Init {
    Cleanup();
    my $ClassObject = LoadClassByAlias( 'Object' );
    my $Class1 = $ClassObject->insertSubClass({ Alias => 'C1' });
    $Class1->insertAttribute({ Alias => 'A1String', Type => 'String', Package => $ATTRIBUTE_PACKAGE });
    $Class1->insertAttribute({ Alias => 'FirstName', Type => 'String', Package => $ATTRIBUTE_PACKAGE });
    $Class1->insertAttribute({ Alias => 'LastName', Type => 'String', Package => $ATTRIBUTE_PACKAGE });
    $Class1->insertAttribute({ Alias => 'Description', Type => 'String', Package => $ATTRIBUTE_PACKAGE });

    $ClassObject->insertObject({ Alias => 'Test', Parent => LoadRootObject() });
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
