use strict;

use Test::More;
use XML::Simple;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::POSTPAY::t::API::TestHelper qw ( LoadTestShop Cleanup Init GetPOSTPAYTestXMLDirectory);
use DE_EPAGES::POSTPAY::t::API::Format qw (heading_1);
use DE_EPAGES::POSTPAY::API::Table::POSTPAYTransactions qw (ExistsPOSTPAYTransaction);
use DE_EPAGES::POSTPAY::API::XML qw (CreatePOSTPAYXML);
use DE_EPAGES::POSTPAY::API::Constants qw (VERSION XMLNSCHECKOUT XMLNSCOMMON );
use DE_EPAGES::Core::API::PerlTools qw (isValid);

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            if( $ENV{SKIP_EXTERNAL_TESTS} ) {
                plan skip_all => 'call POSTPAY Server (SKIP_EXTERNAL_TESTS == 1)';
            }else{
                my $hComparisonData = _loadComparisonData ('SendBasket.xml');
                testPOSTPAYWithUserType('LoadPOSTPAYLineItemPaymentAnonymusUser', $hComparisonData);
                testPOSTPAYWithUserType('LoadPOSTPAYLineItemPaymentRegisteredUser',$hComparisonData);

                my $hExtendedComparisonData = _loadComparisonData ('SendBasketExtended.xml');
                testPOSTPAYWithUserType('LoadPOSTPAYLineItemPaymentAnonymusUserExtended',$hExtendedComparisonData);

                Cleanup();
                done_testing();
            }
        }
    );
}

sub testPOSTPAYWithUserType{
    my ( $UserTypeFunctionName, $hComparisonData) = @_;
    # startUp
    heading_1($UserTypeFunctionName);
    ok( Init(), 'initialization '); # the init() do an internal cleanup() before
    my $LineItemPayment = DE_EPAGES::POSTPAY::t::API::TestHelper->$UserTypeFunctionName;
    my $hResult = _doPOSTPAYRequest($LineItemPayment);
    # test
    _checkHeaderStructure($hResult,$hComparisonData);
    _checkResult($hResult,$hComparisonData);
}

sub _doPOSTPAYRequest {
    my ($LineItemPayment) = @_;
    my $hPostPaySendParams  = _doPOSTPayRequestAndTestTransaction($LineItemPayment);
    my $Shop = LoadTestShop();
    my $XML = CreatePOSTPAYXML($hPostPaySendParams,$Shop);
    _checkAttributes($XML);
    my $hResult = XMLin( $XML, KeepRoot => 0, 'NoAttr'=>1, );
    return $hResult;
}

sub _doPOSTPayRequestAndTestTransaction{
    my ($LineItemPayment) = @_;
    my $hInfo;
    ##no critic qw(ProtectPrivateSubs)
    my $hPostPaySendParams   = DE_EPAGES::POSTPAY::API::Payment::_GetPOSTPAYSendParams(
                                                                    $LineItemPayment,
                                                                    $hInfo,
                                                                    $LineItemPayment->get('TransID'));
    ##use critic
    my $Servlet;
    my $Url = $LineItemPayment->checkoutRequest($Servlet);# <= this is the request

    my $TransactionID =  $LineItemPayment->get('TransID');
    like($Url, qr/http*\:\/\//ms,'return value is url');
    like($Url, qr/$TransactionID/xms,'return url matches basket transaction id');
    ok (ExistsPOSTPAYTransaction($TransactionID),'transaction is in postpaytransaction table');
    return $hPostPaySendParams;
}

sub _loadComparisonData {
    my ($ComparisonFileName) = @_;
    my $Directory  = GetPOSTPAYTestXMLDirectory;
    my $hCorrectResult = XMLin(
        $Directory.$ComparisonFileName,
        KeepRoot => 0,
        'NoAttr'=>1,
    );
    return $hCorrectResult;
}

sub _checkAttributes {
    my ($XML) = @_;
    my $Version = VERSION;
    my $XmlnsCommon = XMLNSCOMMON;
    my $XmlnsCheckout = XMLNSCHECKOUT;

    like(   $XML,
            qr/<\s*submitCartRequest.*xmlns=\"\Q$XmlnsCheckout\E".*>/xms,
            'Attribute xmlns of submitCartRequest element correct');
    like(   $XML,
            qr/<\s*submitCartRequest.*version=\"\Q$Version\E\".*>/xms,
            'Attribute Version of submitCartRequest element correct');
    like(   $XML,
            qr/<\s*submitCartRequest.*xmlns\:common=\"\Q$XmlnsCommon\E".*>/xms,
            'Attribute xmlns:common of submitCartRequest element correct');
    like(   $XML,
            qr/<\s*header.*xmlns=\"\Q$XmlnsCommon\E".*>/xms,
            'Attribute xmln of header element correct');
    return;
}

sub _checkHeaderStructure {
    my ($hResult,$hCorrectResult) = @_;
    is ($hResult->{'header'}->{'login'},$hCorrectResult->{'header'}->{'login'},'header login ok');
    # Just check if password is set, dont need clear text pw of test account on every epages isntallation
    ok (isValid($hResult->{'header'}->{'password'}),'header password set');
    # Just check if language is set because it can differ with the system the test is run on
    ok (isValid($hResult->{'header'}->{'language'}),'header language set');
}

sub _checkResult {
    my ($hResult,$hCorrectResult) = @_;
    is_deeply($hResult->{'shoppingCart'},$hCorrectResult->{'shoppingCart'},'the shoppingCart have the expected structure');
}

RunScript( Sub => \&Main );

1;